;Przykładowy program POOL, możesz go używać,
;modyfikować i rozpowszechniać na licencji GPL.
;Autorzy: R&D Independent Team
;Źródło: www.pool-rnd.com

ht pu                        ;żółwie są ukryte i nie rysują
:pool_cfg,"refresh := false  ;odświeżaniem zajmie się dedykowany stoper

;klasa bazowa:
to linked_node
  let "clicked false
  let "fixed false

  ;lista powiązanych węzłów, musisz ją wypełnić sam!
  let "links []

  let "gravity {0 0}
  let "loss 0.01 ;współczynnik strat (pomaga także utrzymać stabilność)
  let "ampl 0.01 ;stała oddziaływania między węzłami
  let "mass 0.05 ;masa węzła
  let "v 0       ;prędkość węzła
  ht             ;początkowo ukryta ikonka

  ;ustawienia końcowe, funkcja musi być wywołana
  ;na koniec inicjalizacji zaprogramowanej w klasie
  ;dziedziczącej
  to initialized :r :px :py
    "pstart := newarray count :links
    :gravity,1 *= :mass
    :gravity,2 *= :mass
    setxy :px :py ;ustaw początkową pozycję
    setr :r       ;promień żółwia używany domyślnie jako neutralna
                  ;odległość między węzłami
    st            ;pokaż ikonkę kiedy węzeł jest już gotowy
  end

  ;prototyp prostego, przyciągająco-odpychającego oddziaływania
  to fcoeff :d
    op :d - radius
  end

  to sum_forces :fxy [:dist 0] [:coeff 0]
    :fxy,1 := :gravity,1
    :fxy,2 := :gravity,2
    let "pi pos @ :links      ;pozycje połączych węzłów
    foreach "i :pi [
      "dist := distance :i
      "coeff := :ampl * (fcoeff :dist) / :dist
      :fxy,1 += :coeff * (:i,1 - xcor)
      :fxy,2 += :coeff * (:i,2 - ycor)
    ]
  end

  let "force {0 0}
  to onsignalforces
    if !(:fixed || :clicked) [sum_forces :force]
  end

  to on_step_action end ;prototyp dodatkowych akcji, pusty

  to do_step :dt :vd [:vn 0] [:ds 0]
    foreach "i :links [:pstart,repcount := distance :i]

    "dt *= 0.01 ;przedział czasu od ostatniego kroku
    if :dt > 1 ["dt := 1]

    :vd,1 := :vd,1 * :v + :force,1 * :dt / :mass
    :vd,2 := :vd,2 * :v + :force,2 * :dt / :mass
    "vn := sqrt :vd,1^2 + :vd,2^2
    if :vn > 50 ["vn := 50]
    setheaddir :vd
    fd :vn * :dt

    op :vn
  end

  local "pstart  ;odległości między węzłami przed wykonaniem kroku
  to onsignalstep :turtle :data
    if !(:fixed || :clicked) [
      "v := do_step :data movedir
      on_step_action
    ]
  end

  to onsignalloss
    if !(:fixed || :clicked) [
      foreach "i :links [
        "v -= :loss * abs :pstart,repcount - distance :i
      ]
      if :v < 0 ["v := 0]
    ]
  end

  to onmousemove :mousepos
    if :clicked && not :fixed [setpos :mousepos]
  end

  to onturtleclicked
    tto this "clicked := true
  end

  to onmousedown
    eventenable "onmousemove
  end

  to onmouseup
    eventdisable "onmousemove
    "clicked := false
  end
end

